/*
Program: qgrid - Maidenhead grid square utility
Author: ON1MH Johan Maes
e-mail: on1mh@csi.com
Copyright (C) 1999 Johan Maes

This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    see also qgrid.html for further information

*/

/**********************************************************************

	--- Qt Architect generated file ---

	File: grid.cpp
	Last generated: Tue Mar 16 22:11:40 1999

 *********************************************************************/




#include "qgrid.h"
#include "compass.h"
#include <qapp.h>
#include <qmessagebox.h>
#include <ctype.h>
#include <stdio.h>
#include <math.h>
#include <qtooltip.h>
#include <stdlib.h>
#include "version.h"
#define Inherited gridData

grid::grid (QWidget* parent,const char* name):Inherited( parent, name )
{
  clear();
  QToolTip::add( clearbutton, "Clear entries except Home QTH-Locator" );
  QToolTip::add( calcbuttonql, "Convert from QTH-locator to Lat/Long" );
  QToolTip::add( calcbuttonlq, "Convert from Lat/Long to QTH-Locator" );
  QToolTip::add( aboutbutton, "About-Info-Help" );
#ifdef USEMILES
  kmmiles=1;

#else
  kmmiles=0;
#endif
 kmmilesButton->setCurrentItem(kmmiles);  
}


grid::~grid()
{

}

void grid::kmmilesChanged(int val)
{
  kmmiles=val;
  calculateql();
}

void grid::aboutqgrid()
{
  char tempstr[256];
  tempstr[0]=0;
  strcat(tempstr,NAME);
  strcat(tempstr,"version: ");
  strcat(tempstr,VERSION);
  strcat(tempstr," Compiled on ");
  strcat(tempstr,__DATE__);
  strcat(tempstr,"\nsee qgrid.html for more information");
  QMessageBox::about( this,PROGRAMNAME,tempstr);
	
}

void grid::clear()
{
  loca->setText(HOMEQTH);
  locb->setText("");
  latad->setText("");
  latam->setText("");
  latas->setText("");
  latbd->setText("");
  latbm->setText("");
  latbs->setText("");

  lonad->setText("");
  lonam->setText("");
  lonas->setText("");
  lonbd->setText("");
  lonbm->setText("");
  lonbs->setText("");

  distance->setText("");
  bearing->setText("");
}

void grid::strup(char *st)
{
  uint i;
  for (i=0;i<strlen(st);i++)
    {
      st[i]=toupper(st[i]);
    }
}

void grid::calculateql()
{
    char *tempa;
    char *tempb;
    int  d,m,s;
    float latia,lona;
    float latib,lonb;
    float dist,be;
 
    
    tempa=(char *)loca->text();
    strup(tempa);
    if (strlen(tempa)>0)
      {
	if (!validateLocator(tempa))
	  {
	    QMessageBox::warning( this, "Parse Error","Wrong entry for Locator A", "Quit");
	  }
	else
	  {
	    //calculate lat/long
	    calclatlong(tempa,latia, lona);
	    convertToDMS(latia,d,m,s);
	    setint(latad,d);
	    setint(latam,(m>0?m:-m));
 	    setint(latas,(s>0?s:-s));
	    
	    convertToDMS(lona,d,m,s);
	    setint(lonad,d);
	    setint(lonam,(m>0?m:-m));
 	    setint(lonas,(s>0?s:-s));     	    
	  }
      }
    
    tempb=(char *)locb->text();
    strup(tempb);
    
    if (strlen(tempb)>0)
	    {
	    if (!validateLocator(tempb))
	      {
		QMessageBox::warning( this, "Parse Error","Wrong entry for Locator B", "Quit");
	      }
	    else
	      {
		//calculate lat/long
		calclatlong(tempb,latib, lonb);
		convertToDMS(latib,d,m,s);
		setint(latbd,d);
		setint(latbm,(m>0?m:-m));
		setint(latbs,(s>0?s:-s));
		
		convertToDMS(lonb,d,m,s);
		setint(lonbd,d);
		setint(lonbm,(m>0?m:-m));
		setint(lonbs,(s>0?s:-s));
	      }
	    }
    if ((strlen(tempa)>0) && (strlen(tempb)>0))
      {
	dist=calcdistance(latib,lonb,latia,lona,be);
	setint(distance,(int)(dist+0.5));
	setint(bearing,(int)(be+0.5));
	compassframe->set((int)(be+0.5));
	compassframe->repaint(FALSE);	    
      }    
}

void grid::calculatelq()
{
  char tem[7];
  tem[6]=0;
  float latib,latia,lona,lonb;
  int dla,mla,sla,dlb,mlb,slb,dea,mea,sea,deb,meb,seb;
  dla=atoi((char *)latad->text());
  mla=atoi((char *)latam->text());
  sla=atoi((char *)latas->text());
  if(dla<0)
    {
      mla=-mla; sla=-sla;
    }
  dlb=atoi((char *)latbd->text());
  mlb=atoi((char *)latbm->text());
  slb=atoi((char *)latbs->text());
  if(dlb<0)
    {
      mlb=-mlb; slb=-slb;
    }
  dea=atoi((char *)lonad->text());
  mea=atoi((char *)lonam->text());
  sea=atoi((char *)lonas->text());
  if(dea<0)
    {
      mea=-mea; sea=-sea;
    }
  deb=atoi((char *)lonbd->text());
  meb=atoi((char *)lonbm->text());
  seb=atoi((char *)lonbs->text());
  if(deb<0)
    {
      meb=-meb; seb=-seb;
    }
  latia=convertFromDMS(dla,mla,sla);
  latib=convertFromDMS(dlb,mlb,slb);

  lona=convertFromDMS(dea,mea,sea);
  lonb=convertFromDMS(deb,meb,seb);

  calclocator(tem,latia,lona);
  loca->setText(tem);
  
  calclocator(tem,latib,lonb);
  locb->setText(tem);
  //  if ((strlen(tempa)>0) && (strlen(tempb)>0))
      {
	float be,dist;
	dist=calcdistance(latib,lonb,latia,lona,be);
	setint(distance,(int)(dist+0.5));
	setint(bearing,(int)(be+0.5));
	compassframe->set((int)(be+0.5));
	compassframe->repaint(FALSE);	    
      }    
}

void grid::convertToDMS(float x, int &d,int &m, int &s)
{
  float st;
  st=(x>0 ? x+1./7200. : x -1./7200.); // round 1/2 of 1/3600
  d=(int)st;
  st=60*(st-(float)d);
  m=(int)st;
  
  st=60*(st-(float)m);
  s=(int)st; 
}

float grid::convertFromDMS(int d,int m, int s)
{
  return((float)d+(float)m/60.+(float)s/3600.);		
}

void grid::setint(QLineEdit *ql,int x)
{
  char stemp[30];
  sprintf(stemp,"%d",x);
  ql->setText(stemp);
}

void grid::calclatlong(char *st,float &lat,float &lon)
{
  if (st[4]!=0)
    {
      lon= (20.*((float)st[0]-65.) + 2.*((float)st[2]-48.) + 5.*((float)st[4]-65.)/60.)  - 180.;
      lat= (10.*((float)st[1]-65.) +   ((float)st[3]-48.) + 5.*(st[5]-65.)/120.) - 90.;
    }
  else
    {
      lon= (20.*((float)st[0]-65.) + 2.*((float)st[2]-48.))  - 180.;
      lat= (10.*((float)st[1]-65.) +   ((float)st[3]-48.))  - 90.;
    }
}

void grid::calclocator(char *st,float n,float e)
{
  int t1;
  e=e+180.;
  t1=(int)(e/20);
  st[0]=t1+'A';
  e-=(float)t1*20.;
  t1=(int)e/2;
  st[2]=t1+'0';
  e-=(float)t1*2;
  st[4]=(int)(e*12.+0.5)+'A';

  n=n+90.;
  t1=(int)(n/10.);
  st[1]=t1+'A';
  n-=(float)t1*10.;
  st[3]=(int)n+'0';
  n-=(int)n;
  n*=24; // convert to 24 division
  st[5]=(int)(n+0.5)+'A'; 
}

float grid::calcdistance(float latia,float lona,float latib,float lonb,float &be)
{
    // convert to radians
    latia*=(2.*M_PI)/360.;
    lona*=(2.*M_PI)/360.;
    latib*=(2.*M_PI)/360.;
    lonb*=(2.*M_PI)/360.;

    float co = cos(lonb-lona)*cos(latib)*cos(latia) + sin(latib)*sin(latia);
    float ca = acos(co);
    float dx = 6367.*ca;
    if(kmmiles)
      {
	dx/=1.609344;
      }
    
    // If the distance is reasonable, calculate the
    // bearing. Otherwise return it as zero.
    if (dx > 1e-3)
      {	
	float si = sin(lona-lonb)*cos(latia)*cos(latib);
	co = sin(latia) - sin(latib)*cos(ca);
	be = atan2((si > 0 ? si : -si), (co > 0 ? co : -co));
	if (co < 0) be = M_PI - be;
	if (si < 0)	 be = -be;
	if (be < 0) be = be + 2*M_PI;
	be *= 360/(2*M_PI);
      }
    else
      {
	be = 0;
      }
    return dx;
}

bool  grid::validateLocator(const char *st)
{
  
  if ((st[0]>='A')&&(st[0]<='R')&&(st[1]>='A')&&(st[1]<='R'))
    {
      if(isdigit(st[2])&&isdigit(st[3]))
	{
	  if (st[4]==0)
	    {
	      return TRUE;
	    }
	  else if ((st[4]>='A')&&(st[4]<='X')&&(st[5]>='A')&&(st[5]<='X'))
	    {
	      return TRUE;
	    }
	}
    }
  return FALSE;
}



int main( int argc, char **argv )
{
  
  QApplication app(argc,argv); 
  grid  Main;
  Main.setCaption( "Grid Calculator - ON1MH" );    // set title
  Main.setBackgroundColor(QColor(0,128,128) );	// default bg color     
  app.setMainWidget( &Main); 	// this forces a quit of the program
  Main.show();        
  int retval = app.exec();
  return retval;
}








